@@ -1,38 +1,28 @@
+// Copyright 2024 Bjørn Erik Pedersen
+// SPDX-License-Identifier: MIT
+
 package godartsass_test
 
 import (
+	"bytes"
+	crand "crypto/rand"
+	"encoding/base64"
 	"errors"
 	"fmt"
+	"math/rand"
 	"os"
 	"path/filepath"
+	"runtime"
 	"strings"
 	"sync"
 	"testing"
 
 	"github.com/bep/godartsass/v2"
+	"github.com/bep/godartsass/v2/internal/godartsasstesting"
 
 	qt "github.com/frankban/quicktest"
 )
 
-const (
-	sassSample = `nav {
-  ul {
-    margin: 0;
-    padding: 0;
-    list-style: none;
-  }
-
-  li { display: inline-block; }
-
-  a {
-    display: block;
-    padding: 6px 12px;
-    text-decoration: none;
-  }
-}`
-	sassSampleTranspiled = "nav ul {\n  margin: 0;\n  padding: 0;\n  list-style: none;\n}\nnav li {\n  display: inline-block;\n}\nnav a {\n  display: block;\n  padding: 6px 12px;\n  text-decoration: none;\n}"
-)
-
 type testImportResolver struct {
 	name         string
 	content      string
@@ -75,7 +65,7 @@ func TestTranspilerVariants(t *testing.T) {
 		name: "main",
 		content: `
 #main
-    color: blue		
+    color: blue
 `,
 		sourceSyntax: godartsass.SourceSyntaxSASS,
 	}
@@ -84,7 +74,7 @@ func TestTranspilerVariants(t *testing.T) {
 		name   string
 		opts   godartsass.Options
 		args   godartsass.Args
-		expect interface{}
+		expect any
 	}{
 		{"Output style compressed", godartsass.Options{}, godartsass.Args{Source: "div { color: #ccc; }", OutputStyle: godartsass.OutputStyleCompressed}, godartsass.Result{CSS: "div{color:#ccc}"}},
 		{"Enable Source Map", godartsass.Options{}, godartsass.Args{Source: "div{color:blue;}", URL: "file://myproject/main.scss", OutputStyle: godartsass.OutputStyleCompressed, EnableSourceMap: true}, godartsass.Result{CSS: "div{color:blue}", SourceMap: "{\"version\":3,\"sourceRoot\":\"\",\"sources\":[\"file://myproject/main.scss\"],\"names\":[],\"mappings\":\"AAAA\"}"}},
@@ -111,7 +101,7 @@ body
 		{"Error in ImportResolver.Load", godartsass.Options{}, godartsass.Args{Source: "@import \"colors\";", ImportResolver: testImportResolver{name: "colors", failOnLoad: true}}, false},
 		{"Invalid OutputStyle", godartsass.Options{}, godartsass.Args{Source: "a", OutputStyle: "asdf"}, false},
 		{"Invalid SourceSyntax", godartsass.Options{}, godartsass.Args{Source: "a", SourceSyntax: "asdf"}, false},
-		{"Erro logging", godartsass.Options{}, godartsass.Args{Source: `@error "foo";`}, false},
+		{"Error logging", godartsass.Options{}, godartsass.Args{Source: `@error "foo";`}, false},
 	} {
 		test := test
 		c.Run(test.name, func(c *qt.C) {
@@ -210,25 +200,158 @@ div { p { color: $moo; } }`
 	c.Assert(result.CSS, qt.Equals, "content{color:#ccc}div p{color:#f442d1}")
 }
 
+func TestSilenceDeprecations(t *testing.T) {
+	dir1 := t.TempDir()
+	colors := filepath.Join(dir1, "_colors.scss")
+
+	os.WriteFile(colors, []byte(`
+$moo:       #f442d1 !default;
+`), 0o644)
+
+	c := qt.New(t)
+	src := `
+@import "colors";
+div { p { color: $moo; } }`
+
+	var loggedImportDeprecation bool
+	transpiler, clean := newTestTranspiler(c, godartsass.Options{
+		LogEventHandler: func(e godartsass.LogEvent) {
+			if e.DeprecationType == "import" {
+				loggedImportDeprecation = true
+			}
+		},
+	})
+	defer clean()
+
+	result, err := transpiler.Execute(
+		godartsass.Args{
+			Source:              src,
+			OutputStyle:         godartsass.OutputStyleCompressed,
+			IncludePaths:        []string{dir1},
+			SilenceDeprecations: []string{"import"},
+		},
+	)
+	c.Assert(err, qt.IsNil)
+	c.Assert(loggedImportDeprecation, qt.IsFalse)
+	c.Assert(result.CSS, qt.Equals, "div p{color:#f442d1}")
+}
+
+func TestSilenceDependencyDeprecations(t *testing.T) {
+	dir1 := t.TempDir()
+	headings := filepath.Join(dir1, "_headings.scss")
+
+	err := os.WriteFile(headings, []byte(`
+@use "sass:color";
+h1 { color: rgb(color.channel(#aaa, "red", $space: rgb), 0, 0); }
+h2 { color: rgb(color.red(#bbb), 0, 0); } // deprecated
+`), 0o644)
+	if err != nil {
+		t.Fatalf("Failed to write file: %v", err)
+	}
+
+	c := qt.New(t)
+	src := `
+@use "sass:color";
+@use "headings";
+h3 { color: rgb(color.channel(#ccc, "red", $space: rgb), 0, 0); }
+`
+
+	args := godartsass.Args{
+		OutputStyle:  godartsass.OutputStyleCompressed,
+		IncludePaths: []string{dir1},
+	}
+
+	tests := []struct {
+		name                          string
+		src                           string
+		silenceDependencyDeprecations bool
+		expectedLogMessage            string
+		expectedResult                string
+	}{
+		{
+			name:                          "A",
+			src:                           src,
+			silenceDependencyDeprecations: false,
+			expectedLogMessage:            "color.red() is deprecated",
+			expectedResult:                "h1{color:#a00}h2{color:#b00}h3{color:#c00}",
+		},
+		{
+			name:                          "B",
+			src:                           src,
+			silenceDependencyDeprecations: true,
+			expectedLogMessage:            "",
+			expectedResult:                "h1{color:#a00}h2{color:#b00}h3{color:#c00}",
+		},
+		{
+			name:                          "C",
+			src:                           src + "h4 { color: rgb(0, color.green(#ddd), 0); }",
+			silenceDependencyDeprecations: true,
+			expectedLogMessage:            "color.green() is deprecated",
+			expectedResult:                "h1{color:#a00}h2{color:#b00}h3{color:#c00}h4{color:#0d0}",
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			args.Source = tt.src
+			args.SilenceDependencyDeprecations = tt.silenceDependencyDeprecations
+			logMessage := ""
+			transpiler, clean := newTestTranspiler(c, godartsass.Options{
+				LogEventHandler: func(e godartsass.LogEvent) {
+					logMessage = e.Message
+				},
+			})
+			defer clean()
+
+			result, err := transpiler.Execute(args)
+			c.Assert(err, qt.IsNil)
+
+			if tt.expectedLogMessage == "" {
+				c.Assert(logMessage, qt.Equals, "")
+			} else {
+				c.Assert(logMessage, qt.Contains, tt.expectedLogMessage)
+			}
+
+			c.Assert(result.CSS, qt.Equals, tt.expectedResult)
+		})
+	}
+}
+
 func TestTranspilerParallel(t *testing.T) {
 	c := qt.New(t)
 	transpiler, clean := newTestTranspiler(c, godartsass.Options{})
 	defer clean()
 	var wg sync.WaitGroup
 
-	for i := 0; i < 10; i++ {
+	for i := range 20 {
 		wg.Add(1)
 		go func(num int) {
 			defer wg.Done()
-			for j := 0; j < 4; j++ {
+			for range 8 {
 				src := fmt.Sprintf(`
 $primary-color: #%03d;
 
 div { color: $primary-color; }`, num)
 
-				result, err := transpiler.Execute(godartsass.Args{Source: src})
-				c.Check(err, qt.IsNil)
-				c.Check(result.CSS, qt.Equals, fmt.Sprintf("div {\n  color: #%03d;\n}", num))
+				var panicWhen godartsasstesting.PanicWhen
+				if num == 3 {
+					panicWhen = panicWhen | godartsasstesting.ShouldPanicInSendInbound1
+				}
+				if num == 8 {
+					panicWhen = panicWhen | godartsasstesting.ShouldPanicInNewCall
+				}
+				if num == 10 {
+					panicWhen = panicWhen | godartsasstesting.ShouldPanicInSendInbound2
+				}
+				args := godartsass.Args{Source: src}
+				godartsass.TestingApplyArgsSettings(&args, panicWhen)
+				if panicWhen > 0 {
+					c.Check(func() { transpiler.Execute(args) }, qt.PanicMatches, ".*ShouldPanicIn.*")
+				} else {
+					result, err := transpiler.Execute(args)
+					c.Check(err, qt.IsNil)
+					c.Check(result.CSS, qt.Equals, fmt.Sprintf("div {\n  color: #%03d;\n}", num))
+				}
 				if c.Failed() {
 					return
 				}
@@ -253,13 +376,13 @@ func TestTranspilerParallelImportResolver(t *testing.T) {
 
 	var wg sync.WaitGroup
 
-	for i := 0; i < 10; i++ {
+	for i := range 10 {
 		wg.Add(1)
 		go func(i int) {
 			defer wg.Done()
 
-			for j := 0; j < 10; j++ {
-				for k := 0; k < 20; k++ {
+			for j := range 10 {
+				for range 20 {
 					args := godartsass.Args{
 						OutputStyle:    godartsass.OutputStyleCompressed,
 						ImportResolver: createImportResolver(j + i),
@@ -284,15 +407,32 @@ div { p { width: $width; } }`,
 }
 
 func TestTranspilerClose(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		// See https://github.com/sass/dart-sass/issues/2424
+		t.Skip("skipping test on Windows")
+	}
 	c := qt.New(t)
-	transpiler, _ := newTestTranspiler(c, godartsass.Options{})
+	var errBuff bytes.Buffer
+	transpiler, _ := newTestTranspiler(c,
+		godartsass.Options{
+			Stderr: &errBuff,
+			LogEventHandler: func(e godartsass.LogEvent) {
+				fmt.Println("LogEvent:", e)
+			},
+		},
+	)
+
+	defer func() {
+		fmt.Println("Stderr:", errBuff.String())
+	}()
+
 	var wg sync.WaitGroup
 
-	for i := 0; i < 10; i++ {
+	for i := range 10 {
 		wg.Add(1)
 		go func(gor int) {
 			defer wg.Done()
-			for j := 0; j < 4; j++ {
+			for j := range 4 {
 				src := fmt.Sprintf(`
 $primary-color: #%03d;
 
@@ -329,19 +469,51 @@ div { color: $primary-color; }`, gor)
 
 func BenchmarkTranspiler(b *testing.B) {
 	type tester struct {
-		src        string
-		expect     string
+		sources    []string
 		transpiler *godartsass.Transpiler
 		clean      func()
 	}
 
+	const (
+		sassSample = `nav {
+	  ul {
+		margin: 0;
+		padding: 0;
+		list-style: none;
+	  }
+
+	  li { display: inline-block; }
+
+	  a {
+		display: block;
+		padding: 6px 12px;
+		text-decoration: none;
+	  }
+	}`
+	)
+
+	getSassSource := func() string {
+		s := sassSample
+
+		// Append some comment to make it unique.
+		comment := randStr(rand.Intn(1234))
+		s += "\n\n/*! " + comment + " */"
+
+		return s
+	}
+
 	newTester := func(b *testing.B, opts godartsass.Options) tester {
 		c := qt.New(b)
-		transpiler, clean := newTestTranspiler(c, godartsass.Options{})
+		sources := make([]string, b.N)
+		for i := 0; i < b.N; i++ {
+			sources[i] = getSassSource()
+		}
+		transpiler, clean := newTestTranspiler(c, opts)
 
 		return tester{
 			transpiler: transpiler,
 			clean:      clean,
+			sources:    sources,
 		}
 	}
 
@@ -349,20 +521,16 @@ func BenchmarkTranspiler(b *testing.B) {
 		defer t.clean()
 		b.ResetTimer()
 		for n := 0; n < b.N; n++ {
-			result, err := t.transpiler.Execute(godartsass.Args{Source: t.src})
+			_, err := t.transpiler.Execute(godartsass.Args{Source: t.sources[n]})
 			if err != nil {
 				b.Fatal(err)
 			}
-			if result.CSS != t.expect {
-				b.Fatalf("Got: %q\n", result.CSS)
-			}
+
 		}
 	}
 
 	b.Run("SCSS", func(b *testing.B) {
 		t := newTester(b, godartsass.Options{})
-		t.src = sassSample
-		t.expect = sassSampleTranspiled
 		runBench(b, t)
 	})
 
@@ -370,33 +538,26 @@ func BenchmarkTranspiler(b *testing.B) {
 	b.Run("Start and Execute", func(b *testing.B) {
 		for n := 0; n < b.N; n++ {
 			t := newTester(b, godartsass.Options{})
-			t.src = sassSample
-			t.expect = sassSampleTranspiled
-			result, err := t.transpiler.Execute(godartsass.Args{Source: t.src})
+			_, err := t.transpiler.Execute(godartsass.Args{Source: t.sources[n]})
 			if err != nil {
 				b.Fatal(err)
 			}
-			if result.CSS != t.expect {
-				b.Fatalf("Got: %q\n", result.CSS)
-			}
 			t.transpiler.Close()
 		}
 	})
 
 	b.Run("SCSS Parallel", func(b *testing.B) {
 		t := newTester(b, godartsass.Options{})
-		t.src = sassSample
-		t.expect = sassSampleTranspiled
+
 		defer t.clean()
 		b.RunParallel(func(pb *testing.PB) {
+			n := 0
 			for pb.Next() {
-				result, err := t.transpiler.Execute(godartsass.Args{Source: t.src})
+				_, err := t.transpiler.Execute(godartsass.Args{Source: t.sources[n]})
 				if err != nil {
 					b.Fatal(err)
 				}
-				if result.CSS != t.expect {
-					b.Fatalf("Got: %q\n", result.CSS)
-				}
+				n++
 			}
 		})
 	})
@@ -408,7 +569,7 @@ func TestVersion(t *testing.T) {
 	version, err := godartsass.Version(getSassEmbeddedFilename())
 	c.Assert(err, qt.IsNil)
 	c.Assert(version, qt.Not(qt.Equals), "")
-	c.Assert(strings.HasPrefix(version.ProtocolVersion, "2."), qt.IsTrue)
+	c.Assert(strings.HasPrefix(version.ProtocolVersion, "3."), qt.IsTrue, qt.Commentf("got: %q", version.ProtocolVersion))
 }
 
 func newTestTranspiler(c *qt.C, opts godartsass.Options) (*godartsass.Transpiler, func()) {
@@ -430,3 +591,10 @@ func getSassEmbeddedFilename() string {
 
 	return "sass"
 }
+
+func randStr(len int) string {
+	buff := make([]byte, len)
+	crand.Read(buff)
+	str := base64.StdEncoding.EncodeToString(buff)
+	return str[:len]
+}
